@@ -40,12 +40,14 @@
 #include <QRegExpValidator>
 #include <QSettings>
 #include <QStringList>
+#include <QTranslator>
 #include <QVariant>
 #include <QWhatsThis>
 
 #include "qdjviewprefs.h"
 #include "qdjview.h"
 #include "qdjvu.h"
+#include "djview.h"
 
 
 
@@ -145,6 +147,7 @@ QDjViewPrefs::QDjViewPrefs(void)
     showTextLabel(false),
     invertLuminance(false),
     mouseWheelZoom(false),
+    restrictOverride(false),
     modifiersForLens(Qt::ControlModifier|Qt::ShiftModifier),
     modifiersForSelect(Qt::ControlModifier),
     modifiersForLinks(Qt::ShiftModifier),
@@ -353,6 +356,10 @@ QDjViewPrefs::load()
     invertLuminance = s.value("invertLuminance").toBool();
   if (s.contains("mouseWheelZoom"))
     mouseWheelZoom = s.value("mouseWheelZoom").toBool();
+  if (s.contains("language"))
+    languageOverride = s.value("language").toString();
+  if (s.contains("restrictOverride"))
+    restrictOverride = s.value("restrictOverride").toBool();
   if (s.contains("modifiersForLens"))
     modifiersForLens 
       = stringToModifiers(s.value("modifiersForLens").toString());
@@ -362,7 +369,7 @@ QDjViewPrefs::load()
   if (s.contains("modifiersForLinks"))
     modifiersForLinks 
       = stringToModifiers(s.value("modifiersForLinks").toString());
-
+  
   if (s.contains("thumbnailSize"))
     thumbnailSize = s.value("thumbnailSize").toInt();
   if (s.contains("thumbnailSmart"))
@@ -446,6 +453,8 @@ QDjViewPrefs::save(void)
   s.setValue("modifiersForLens", modifiersToString(modifiersForLens));
   s.setValue("modifiersForSelect", modifiersToString(modifiersForSelect));
   s.setValue("modifiersForLinks", modifiersToString(modifiersForLinks));
+  s.setValue("language", languageOverride);
+  s.setValue("restrictOverride", restrictOverride);
 
   s.setValue("thumbnailSize", thumbnailSize);
   s.setValue("thumbnailSmart", thumbnailSmart);
@@ -822,22 +831,13 @@ QDjViewPrefsDialog::QDjViewPrefsDialog()
              "</html>") );
   
   setHelp(d->ui.advancedTab,
-          tr("<html><b>Caches.</b>"
-             "<br>The <i>pixel cache</i> stores image data"
-             " located outside the visible area."
-             " This cache makes panning smoother."
-             " The <i>decoded page cache</i> contains partially"
-             " decoded pages. It provides faster response times"
-             " when navigating a multipage document or when returning"
-             " to a previously viewed page. Clearing this cache"
-             " might be useful to reflect a change in the page"
-             " data without restarting the DjVu viewer."
-
-             "<p><b>Miscellaneous.</b>"
-             "<br>Forcing a manual color correction can be useful"
-             " when using ancient printers."
-             " The advanced features check box enables a small number"
-             " of additional menu entries useful for authoring DjVu files."
+          tr("<html><b>Advanced.</b>"
+             "<br>You can override the default interface language, "
+             "disable the page animations, or enable additional "
+             "menu entries that are useful for authoring DjVu files."
+             "You can also disable the printing or saving restrictions "
+             "dictated by certain web sites. The manual color "
+             "correction can be useful with old printers."
              "</html>"));
 
   setHelp(d->ui.networkTab,
@@ -847,6 +847,16 @@ QDjViewPrefsDialog::QDjViewPrefsDialog()
              " a djvu document through a http url."
              " The djview plugin always uses the proxy"
              " settings of the web browser."
+             "<p><b>Cache settings.</b>"
+             "<br>The <i>pixel cache</i> stores image data"
+             " located outside the visible area."
+             " This cache makes panning smoother."
+             " The <i>decoded page cache</i> contains partially"
+             " decoded pages. It provides faster response times"
+             " when navigating a multipage document or when returning"
+             " to a previously viewed page. Clearing this cache"
+             " might be useful to reflect a change in the page"
+             " data without restarting the DjVu viewer."""
              "</html>") );
 }
 
@@ -856,11 +866,13 @@ QDjViewPrefsDialog::connectModified(QWidget *w)
 {
   if (w->inherits("QComboBox"))
     connect(w, SIGNAL(editTextChanged(QString)), this, SLOT(setModified()));
-  else if (w->inherits("QSpinBox"))
+  if (w->inherits("QComboBox"))
+    connect(w, SIGNAL(currentIndexChanged(int)), this, SLOT(setModified()));
+  if (w->inherits("QSpinBox"))
     connect(w, SIGNAL(valueChanged(int)), this, SLOT(setModified()));
-  else if (w->inherits("QAbstractButton"))
+  if (w->inherits("QAbstractButton"))
     connect(w, SIGNAL(toggled(bool)), this, SLOT(setModified()));
-  else if (w->inherits("QAbstractSlider"))
+  if (w->inherits("QAbstractSlider"))
     connect(w, SIGNAL(valueChanged(int)), this, SLOT(setModified()));
   QObject *child;
   foreach(child, w->children())
@@ -921,23 +933,66 @@ QDjViewPrefsDialog::load(QDjView *djview)
       d->ui.proxyPasswordLineEdit->setText(url.password());
     }
   d->ui.proxyCheckBox->setChecked(proxy);
-  // 6- advanced tab
   qreal k256 = 256 * 1024;
   qreal k1024 = 1024 * 1024;
   d->ui.pixelCacheSpinBox->setValue(qRound(prefs->pixelCacheSize / k256));
   d->ui.pageCacheSpinBox->setValue(qRound(prefs->cacheSize / k1024));
+  // 6- advanced tab
   double pgamma = prefs->printerGamma;
+  loadLanguageComboBox(prefs->languageOverride);
+  d->ui.advancedCheckBox->setChecked(prefs->advancedFeatures);
+  d->ui.animationCheckBox->setChecked(prefs->enableAnimations);
+  d->ui.textLabelCheckBox->setChecked(prefs->showTextLabel);
+  d->ui.restrictOverrideCheckBox->setChecked(prefs->restrictOverride);
   d->ui.printerManualCheckBox->setChecked(pgamma > 0);
   d->ui.printerGammaSpinBox->setValue((pgamma > 0) ? pgamma : 2.2);
-  d->ui.animationCheckBox->setChecked(prefs->enableAnimations);
-  d->ui.advancedCheckBox->setChecked(prefs->advancedFeatures);
-  d->ui.textLabelCheckBox->setChecked(prefs->showTextLabel);
   // no longer modified
   setWindowModified(false);
   d->ui.applyButton->setEnabled(false);
 }
 
 
+void
+QDjViewPrefsDialog::loadLanguageComboBox(QString lang)
+{
+  // supported languages
+  static const char *languages[] = { 
+    "cs","de","en","fr","it","ja","ru","uk","zh_cn","zh_tw",0 } ;
+  // get application
+  QComboBox *cb = d->ui.languageComboBox;
+  QCheckBox *db = d->ui.languageCheckBox;
+  QDjViewApplication *app = qobject_cast<QDjViewApplication*>(qApp);
+  db->setEnabled(app != 0);
+  cb->clear();
+  // prepare combobox
+  static struct { const char *src; const char *com; } thisLang[] = {
+    QT_TRANSLATE_NOOP3("Generic", "thisLanguage", "Name of THIS language") };
+  QString myLanguage = QApplication::translate("Generic", thisLang[0].src);
+  if (myLanguage == thisLang[0].src)
+    myLanguage = "English";
+  int index = -1;
+  if (app) 
+    for (int i=0; languages[i]; i++)
+      {
+        QString nlang = languages[i];
+        QString xlang = QString::null;
+        QTranslator *trans = new QTranslator();
+        if (app->loadTranslator(trans, "djview", QStringList(nlang)))
+          xlang = trans->translate("Generic", thisLang[0].src);
+        else if (nlang == "en" || nlang.startsWith("en"))
+          xlang = "English";
+        if (lang == nlang && xlang.size() > 0)
+          index = cb->count();
+        if (index < 0 && xlang == myLanguage)
+          index = cb->count();
+        if (xlang.size() > 0)
+          cb->addItem(xlang, nlang);
+        delete trans;
+      }
+  db->setChecked(index >= 0 && lang.size() > 0);
+  cb->setCurrentIndex(index);
+}
+
 void
 QDjViewPrefsDialog::apply()
 {
@@ -975,15 +1030,22 @@ QDjViewPrefsDialog::apply()
       prefs->proxyUrl.setUserName(d->ui.proxyUserLineEdit->text());
       prefs->proxyUrl.setPassword(d->ui.proxyPasswordLineEdit->text());
     }
-  // 6- advanced tab
   prefs->pixelCacheSize = d->ui.pixelCacheSpinBox->value() * 256 * 1024;
   prefs->cacheSize = d->ui.pageCacheSpinBox->value() * 1024 * 1024;
-  prefs->printerGamma = 0;
-  if (d->ui.printerManualCheckBox->isChecked())
-    prefs->printerGamma = d->ui.printerGammaSpinBox->value();
+  // 6- advanced tab
+  prefs->languageOverride = QString::null;
+  if (d->ui.languageCheckBox->isChecked()) 
+    {
+      int n = d->ui.languageComboBox->currentIndex();
+      prefs->languageOverride = d->ui.languageComboBox->itemData(n).toString();
+    }
   prefs->enableAnimations = d->ui.animationCheckBox->isChecked();
   prefs->advancedFeatures = d->ui.advancedCheckBox->isChecked();
   prefs->showTextLabel = d->ui.textLabelCheckBox->isChecked();
+  prefs->restrictOverride = d->ui.restrictOverrideCheckBox->isChecked();
+  prefs->printerGamma = 0;
+  if (d->ui.printerManualCheckBox->isChecked())
+    prefs->printerGamma = d->ui.printerGammaSpinBox->value();
   // broadcast change
   setWindowModified(false);
   d->ui.applyButton->setEnabled(false);
@@ -1029,6 +1091,7 @@ QDjViewPrefsDialog::reset()
   // 6- advanced tab
   d->ui.pixelCacheSpinBox->setValue(1);
   d->ui.pageCacheSpinBox->setValue(10);
+  d->ui.languageCheckBox->setChecked(false);
   d->ui.printerManualCheckBox->setChecked(false);
   d->ui.printerGammaSpinBox->setValue(2.2);
   d->ui.advancedCheckBox->setChecked(false);
